#pragma once

#include <d3dx10.h>
#include <string>
#include <vector>
#include "ResourceManagement/Mesh/TFStructs.h"

namespace Marabunta
{
	namespace Render
	{
		struct CompiledMaterial;

		struct GeometryParameters
		{
			std::string MeshName;
			Marabunta::Render::CompiledMaterial* Material;

			GeometryParameters(){}

			GeometryParameters(std::string mesh, Marabunta::Render::CompiledMaterial* mat)
			{
				MeshName = mesh;
				Material = mat;
			}
		};
	}
	namespace SceneManagement
	{
		class Camera;
		class SpatialGraph;	
		class Geometry;
	}
	namespace Mesh
	{
		struct TFBoundingBox;
		class MeshModel;

		class BaseMesh
		{
		public:
			virtual void SetupEffectVariables();
			virtual void SetEffectVariables(D3DXMATRIX* world);
			void SetCamera(SceneManagement::Camera* cam);

			virtual void SetLayout();
			virtual void SetMaterial(Marabunta::Render::CompiledMaterial* mat);

			std::vector<D3DXVECTOR3> GetLocators() { return _locators; }

			void UpdateDraw(SceneManagement::SpatialGraph* scene);
			void Render();

		protected:
			//abstract class
			BaseMesh(void);
			virtual ~BaseMesh(void);
			BaseMesh(Marabunta::Render::CompiledMaterial* mat);

			Marabunta::Render::CompiledMaterial*	_material;
			ID3D10Device*		_device;
			MeshModel*			_mesh;
			TFBoundingBox		_meshBoundingBox;
			
			//matrices for the card
			ID3D10EffectMatrixVariable* _worldMatrix;
			ID3D10EffectMatrixVariable* _viewMatrix;
			ID3D10EffectMatrixVariable* _projectionMatrix;

			std::vector<D3DXVECTOR3> _locators;

		protected:
			bool CreateLayout();

			friend Marabunta::SceneManagement::Geometry;
		};
	}
}