#pragma once

#include <string>
#include <vector>
#include <d3dx10.h>
#include "TFStructs.h"
#include "ResourceManagement/Mesh/BaseMesh.h"

namespace Marabunta
{
	class MarabuntaCore;
	namespace Mesh
	{
		class MeshModel;
		class StaticMesh;
		class AnimatedMesh;

		struct AnimatedMeshResource
		{
			std::string Name;
			MeshModel* Mesh;
			TFAnimatedMesh* AnimatedMesh;
			AnimatedMeshResource()
			{
				Name = "";
				Mesh = 0;
				AnimatedMesh = 0;
			}
		};

		struct StaticMeshResource
		{
			std::string Name;
			MeshModel* Mesh;
			TFStaticMesh* StaticMesh;
			StaticMeshResource()
			{
				Name = "";
				Mesh = 0;
				StaticMesh = 0;
			}
		};

		class MeshManager
		{
			friend class Marabunta::MarabuntaCore;
		public:
			void CreateMesh(Marabunta::Render::GeometryParameters geom, StaticMesh** mesh);
			void CreateMesh(Marabunta::Render::GeometryParameters geom, AnimatedMesh** mesh);
			void CleanUp();
		private:
			MeshManager();
			~MeshManager(void);
			
			ID3D10Device* _device;
			std::vector<StaticMeshResource> _staticModels;
			std::vector<AnimatedMeshResource> _animatedModels;
			std::vector<AnimatedMesh*> _animatedMeshes;
			std::vector<StaticMesh*> _staticMeshes;

		};
	}
}
