#pragma once
#include "Node.h"

namespace Marabunta
{
	namespace SceneManagement
	{
		class Spatial;

		class Camera : public Node
		{
		public:
			Camera(void);
			virtual ~Camera(void);
			void CreateCamera();
			virtual void UpdateGS(float dt);
			
			void SetLookAt(float aX, float aY, float aZ, bool update = false);
			void SetLookAt(D3DXVECTOR3 aVec);
			void SetLookAt(Spatial* s);

			void SetUp(D3DXVECTOR3 up);

			void TranslateLookAt(D3DXVECTOR3 delta);

			float GetLookAtX();
			float GetLookAtY();
			float GetLookAtZ();
			D3DXVECTOR3 GetUp();
			D3DXVECTOR3 GetLookAt();
			D3DXVECTOR2 GetFOV();

		private:
			void BuildViewMatrix();
			void BuildProjectionMatrix();

		public:
			D3DXMATRIX* viewMatrix;
			D3DXMATRIX* projectionMatrix;

		private:
			//note that a camera only has 2 children.
			//DO NOT ADD MORE!
			//0. up
			//1. lookat
			static const int UP = 0;
			static const int LOOK_AT = 1;
			Spatial* pLookAtTarget;
			bool useLookAtTarget;

			//for internal performance purposes
			D3DXVECTOR3 position;
			D3DXVECTOR3 lookat;
			D3DXVECTOR3 up;
			D3DXVECTOR2 fov;

			
			static const float NEAR_CLIP_PLANE;
			static const float FAR_CLIP_PLANE;
		};
	}
}