#pragma once

#include <d3dx10.h>

namespace Marabunta
{
	namespace SceneManagement
	{
		//to let us distinguish between spatial types without using the slow typeinfo
		enum ElementType
		{
			ET_Spatial,
			ET_Node,
			ET_Geometry,
			ET_Camera,
			ET_Terrain,
			ET_WorldObject
		};

		class Scene;
		class SpatialGraph;
		class BoundingVolume;

		class Spatial
		{
		public:
			Spatial(void);
			virtual ~Spatial(void);

			void Initialize();
			
			void SetParent(Spatial* p);

			//geometry state
			virtual void UpdateGS(float dt);
			//bound state
			virtual BoundingVolume* UpdateBS();

			//draw
			virtual void UpdateDraw();

			//transformation modifiers and accessors
			//translation
			void Translate(const D3DXVECTOR3 delta);
			void SetTranslation(const D3DXVECTOR3 newPos);
			void GetTranslation(D3DXVECTOR3 &outPos);
			D3DXVECTOR3 GetTranslation();
			
			//rotation
			void Rotate(const D3DXVECTOR3 delta);
			void SetRotation(const D3DXVECTOR3 newRot);
			void GetRotation(D3DXVECTOR3 &outRot);
			D3DXVECTOR3 GetRotation();

			//scale
			void Scale(const D3DXVECTOR3 delta);
			void SetScale(const D3DXVECTOR3 newScale);
			void GetScale(D3DXVECTOR3 &outScale);
			D3DXVECTOR3 GetScale();

			D3DXMATRIX* GetWorld();
			D3DXVECTOR3 GetWorldTranslation();

			virtual BoundingVolume* GetBound();
			bool IsDirty();

			//Bounding volume state options
			enum CullingMode
			{
				CULL_DYNAMIC, //culled if it fails the frustum test
				CULL_ALWAYS,
				CULL_NEVER
			};
			CullingMode Culling;

		protected:
			void BuildLocal();
			void UpdateWorldBound();
			void PropagateBoundToRoot();

		protected:
			ElementType _type;

			//local transformation
			D3DXVECTOR3 _scale;
			D3DXVECTOR3 _rotation;
			D3DXVECTOR3 _translation;
			
			D3DXMATRIX _local;
			D3DXMATRIX _world;

			//support for hierarchical scene graph
			Spatial* _parent;

			//has it been updated and it needs new matrices?
			bool _dirtyTransform;

			//internally used variables for performance
			D3DXMATRIX _scaleTemp;
			D3DXMATRIX _rotationTemp;
			D3DXMATRIX _translationTemp;

			//bounding volume
			BoundingVolume* _boundingVolume;
		};
	}
}